<?php

class analytics_api {

    protected $auth;
    
    public $accounts;
    
    public $ids;
    
    public $error = '';
    
    /**
     * Authentication URL generation
     *
     * @return url
     */
    public function get_auth_url(){
        $authURL = '';
        $ga_oauth_client_id  = get_option( GA_OPT_CLIENT_ID );
        if (empty($ga_oauth_client_id)){
            return $authURL;
        }
        
        $authURL = GA_BASE_URL.
                   '?scope='.urlencode(GA_SCOPE_URL).
                   '&redirect_uri=' . urlencode(GA_CALLBACK_URL) .
                   '&response_type=code' .
                   '&client_id=' . $ga_oauth_client_id;
        
        return $authURL;
    }
    
    /**
     * Set the Access token
     *
     * @param $code
     * @param $client_id
     * @param $client_secret
     * @param $callback_url
     * @return boolean
     */
    public function set_access_token($code, $client_id, $client_secret, $callback_url){
        /*
         *  Get Access Token 
         */
        $url = 'https://accounts.google.com/o/oauth2/token';
        $params = array(
            'code'          => $code,
            'client_id'     => $client_id,
            'client_secret' => $client_secret,
            'redirect_uri'  => $callback_url,
            'grant_type'    => 'authorization_code'
        );
        
        // Send POST
        $options = array('http' => array(
            'method' => 'POST',
            'content' => http_build_query($params)
        ));
        $res = @file_get_contents($url, false, stream_context_create($options));
        if(!$res){
            return false;
        }

        // The response acquisition
        $token = json_decode($res, true);
        if(isset($token['error'])){
            return false;
        }
        
        $access_token = $token['access_token'];
        //var_dump($access_token);exit;
        $_SESSION['access_token'] = $access_token;
        return true;
    }
    
    /**
     * Checking whether the access token is set
     *
     * @return string or boolean false
     */
    public function chk_access_token(){
        if (!isset($_SESSION['access_token']) || empty($_SESSION['access_token'])){
            return false;
        }
        
        return $_SESSION['access_token'];
    }
    
    /**
     * Calls the API using the parameters passed in and returns the data in an array.
     *
     * @param $dimension
     * @param $metric
     * @param $sort
     * @param $start
     * @param $end
     * @param $max_results
     * @param $start_index
     * @return Object
     */
    public function data($dimension, $metric, $sort = false, $start = false, $end = false, $max_results = 10, $start_index = 1) {
        
        if(!$sort) $sort = "-$metric";
        if(!$start) $start = date('Y-m-d', strtotime('1 month ago'));
        if(!$end) $end = date('Y-m-d', strtotime('yesterday'));
        
        $access_token = $this->chk_access_token();
        
        $url = "https://www.googleapis.com/analytics/v3/data/ga".
               "?dimensions=$dimension".
               "&ids=ga:".$this->ids.
               "&metrics=$metric".
               "&sort=$sort".
               "&start-date=$start".
               "&end-date=$end".
               "&max-results=$max_results".
               "&start-index=$start_index".
               "&access_token={$access_token}";

        $json = $this->call($url);
        
        $obj_data = json_decode($json);
        
        return $obj_data->rows;
    }

    /**
     * Calls an API function using the url passed in and returns either the XML returned from the
     * call or false on failure
     *
     * @param $url 
     * @return string or boolean false
     */
    public function call($url) {
        $ch = $this->curl_init($url);
        $output = curl_exec($ch);
        $info = curl_getinfo($ch);
        curl_close($ch);

        // set return value to a default of false; it will be changed to the return string on success
        $return = false;
        
        if($info['http_code'] == 200) {
            $return = $output;
            $this->error = "";
        }else if($info['http_code'] == 400) {
            $this->error = "400 Bad Request";
        }else if($info['http_code'] == 401) {
            unset($_SESSION['access_token']);
            $this->error = array("401 Unauthorized");
        }else {
            $this->error = "{$info['http_code']} Unknown error";
        }

        return $return;
        
    }
    
    /**
    * Returns an instance from curl_init with all the commonly needed properties set.
    * 
    * @param $url string The $url to open
    * @return object
    */
    protected function curl_init($url) {
        
        $ch = curl_init();
        
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
        
        return $ch;
        
    }
}
