<?php
/*
Plugin Name: G Analytics
Plugin URI: http://ivystar.jp/software/g-analytics/
Description: G Analyticst is a plug-in for WordPress, which can be used to check the Google Analytics site analysis results in the management screen of WordPress.
Version: 2.0.5
Author: Ivystar
Author URI: http://ivystar.jp/
*/

/*  Copyright 2009-2015 Ivystar (email : info@ivystar.jp)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/***************************************************
 * Want to enable output buffering
 ***************************************************/
ob_start();

/***************************************************
 * Loading file translation
 ***************************************************/
load_plugin_textdomain('g_analytics',false, dirname(plugin_basename(__FILE__)).'/languages/');

/***************************************************
 * Read include files
 ***************************************************/
require_once('constant.php');
require_once('analytics_api.php');
require_once('forms/header.php');
require_once('forms/table.php');
require_once('forms/top.php');
require_once('forms/setup.php');

/***************************************************
 * Add Action
 ***************************************************/
add_action('init',                'ga_session_start');
add_action('admin_print_scripts', 'ga_script');
add_action('admin_print_styles',  'ga_style');
add_action('admin_menu',          'ga_sidemenu');

/***************************************************
 * Loading external scripts
 ***************************************************/
function ga_script(){
    wp_enqueue_script( 'excanvas_pack', '/wp-content/plugins/g_analytics/js/flot/excanvas.min.js');
    wp_enqueue_script( 'flot',          '/wp-content/plugins/g_analytics/js/flot/jquery.flot.js');
}

/***************************************************
 * Loading external style sheet
 ***************************************************/
function ga_style(){
    wp_enqueue_style('com_design', '/wp-content/plugins/g_analytics/css/design.css');
}

/***************************************************
 * Set Side Menu
 ***************************************************/
function ga_sidemenu() {
    $optval_rd_auth  = get_option( GA_OPT_RD_AUTH );
    if (empty($optval_rd_auth)){
        $optval_rd_auth = 8;
    }
    
    add_menu_page('G Analytics', 'G Analytics', $optval_rd_auth, 'GAnalytics', 'ga_main', GA_MY_PATH.'menu_icon.png');
    add_submenu_page('GAnalytics', 'G Analytics', __('Access analysis', 'g_analytics'), $optval_rd_auth, "GAnalytics", 'ga_main');
    add_submenu_page('GAnalytics', __('Setup', 'g_analytics'), __('Setup', 'g_analytics'), $optval_rd_auth, "GAnalyticsSetup", 'ga_setup');
}


/***************************************************
 * Session Start
 ***************************************************/
function ga_session_start(){
    if (!isset($_SESSION)) {
        session_start();
    }
}

/***************************************************
 * Main function
 ***************************************************/
function ga_main() {

    // Module Check
    $arr_mc = ga_module_check();
    if ( count($arr_mc) > 0 ){
        ga_set_errmsg($arr_mc, true);
        return;
    }
    
    // Set Google Analytics API
    $api = new analytics_api();
    $authURL = $api->get_auth_url();
    if (empty($authURL)){
        ga_setup('Authentication url can not be retrieved correctly. Please check your settings.', false);
        exit;
    }
    
    // In the case of call-back from the OAuth 2.0
    if(isset($_REQUEST['mode']) && $_REQUEST['mode'] === 'callback'){
        // Get the code at the time of call-back
        if(!isset($_REQUEST['code']) || $_REQUEST['code'] === ''){
            ga_setup('Callback code could not be acquired', false);
            ob_end_flush();
            exit;
        }
        $code = $_REQUEST['code'];
        
        // Get set value
        $optval_client_id     = get_option( GA_OPT_CLIENT_ID );
        $optval_client_secret = get_option( GA_OPT_CLIENT_SECRET );
        
        // Set Access Token
        $res = $api->set_access_token($code, $optval_client_id, $optval_client_secret, GA_CALLBACK_URL);
        if (!$res){
            ga_set_errmsg(array('Access token could not be set'), true);
            ob_end_flush();
            return;
        }
        
        // Go to Main processing
        header('Location:'.$_SERVER['PHP_SELF'].'?page=GAnalytics&mode=main');
        
    // Main processing
    }else if(isset($_REQUEST['mode']) && $_REQUEST['mode'] === 'main'){
        // Check Access token
        if (!$api->chk_access_token()){
            
            if (empty($authURL)){
                ga_setup('Authentication for the URL could not be retrieved', false);
                ob_end_flush();
                exit;
            }else{
                header('Location:'.$authURL);
            }
        }
        
        // Flush (send) the output buffer and turn off output buffering
        ob_end_flush();
        
        // Set Html Header
        ga_html_header(true);
        
        // Get set value
        $optval_ga_id = get_option( GA_OPT_ID );
        $optval_period = get_option( GA_OPT_PERIOD );
        if (empty($optval_period)) $optval_period = '1month';
        $p_date = ga_get_date($optval_period);
        
        // Set Ids
        $api->ids = $optval_ga_id;
        
        $top_flag = false;
        $ga_page = $_GET['ga_page'];
        switch($ga_page){
            case 'daily_visitors':
                
                $h3_element = 'Daily Visitors';
                $first_title = 'Date';
                $element_link = false;
                $metric .= 'ga:visits,';
                $metric .= 'ga:pageviews,';
                $metric .= 'ga:visitors,';
                $metric .= 'ga:timeOnPage,';
                $metric .= 'ga:bounces,';
                $metric .= 'ga:entrances';
                
                $arr_data = @$api->data('ga:date', 
                                        $metric, 
                                        'ga:date', 
                                        $p_date['start_date'], 
                                        $p_date['end_date'], 
                                        $p_date['max_results']);
                break;
            case 'accessed_page':
                
                $h3_element = 'Accessed by page number';
                $first_title = 'Title';
                $element_link = GA_PAGE_TITLE;
                $metric .= 'ga:pageviews,';
                $metric .= 'ga:bounces,';
                $metric .= 'ga:entrances';
                
                $arr_data = @$api->data('ga:pagePath',
                                        $metric,
                                        false,
                                        $p_date['start_date'],
                                        $p_date['end_date'],
                                        100);
                if (is_array($arr_data)){
                    $siteurl = get_option('siteurl');
                    foreach($arr_data as $key => $val){
                        if (preg_match('/preview=true/', $key)){
                            $arr_data[$key][4] .= "(".__('Preview', 'g_analytics').")";
                        }else{
                            $arr_data[$key][4] = ga_get_title($val[0]);
                        }
                    }
                }
                $metric .= ",".GA_PAGE_TITLE;
                break;
                
            case 'referrer';
                
                $h3_element = 'Referrer';
                $first_title = 'Referrer';
                $element_link = false;
                $metric .= 'ga:visits,';
                $metric .= 'ga:pageviews';
                
                $arr_data = @$api->data('ga:source',
                                        $metric,
                                        false, 
                                        $p_date['start_date'],
                                        $p_date['end_date'],
                                        100);
                
                break;
                
            case 'keyword';
                
                $h3_element = 'Keyword';
                $first_title = 'Keyword';
                $element_link = false;
                $metric .= 'ga:visits,';
                $metric .= 'ga:pageviews';
                
                $arr_data = @$api->data('ga:keyword',
                                        $metric,
                                        false, 
                                        $p_date['start_date'],
                                        $p_date['end_date'],
                                        100);
                
                break;
                
            case 'browser';
                
                $h3_element = 'Browser';
                $first_title = 'Browser';
                $element_link = false;
                $metric .= 'ga:visits,';
                $metric .= 'ga:pageviews';
                
                $arr_data = @$api->data('ga:browser',
                                        $metric,
                                        false, 
                                        $p_date['start_date'],
                                        $p_date['end_date'],
                                        100);
                
                break;
            case 'top';
            default:
                $top_flag = true;
                ga_get_top($api, $optval_ga_id, $optval_period);
                break;
        }
        
        // Other than the top page, and outputs an error
        if (!$top_flag){
            $api_err = $api->error;
            if (!empty($api_err)){
                ga_set_errmsg($api_err, true);
            }
        }
        
        // Output HTML
        if (!empty($ga_page) && $ga_page != 'top') {
            $arr_metric = explode(",", $metric);
            $arr_data = ga_get_make_analytics_data($arr_data, $arr_metric, $first_title);
            
            echo '<h3>';
            _e($h3_element, 'g_analytics');
            echo '</h3>';
            echo '<p>';
            ga_set_table_form($arr_data, $element_link);
            echo '</p>';
        }
    }else{
        header('Location:'.$authURL);
    }
}
/********** Function ga_main() END **********/

/***************************************************
 * Output Setup HTML
 ***************************************************/
function ga_setup($msg = "", $period_flag = false){

    ga_html_header($period_flag);
    ga_html_setup($msg);
    
}
/********** Function ga_setup() END **********/


/***************************************************
 * Module Check
 *--------------------------------------------------
 * @return array
 ***************************************************/
function ga_module_check(){

    $arr_msg = array();
    
    if ( ! (function_exists('curl_init') && extension_loaded('curl')) ) {
        $arr_msg[] = __('"CURL" no module.', 'g_analytics');
    }

    if ( ! (class_exists('DOMDocument') && extension_loaded('dom')) ) {
        $arr_msg[] = __('"DOM" no module.', 'g_analytics');
    }
    
    return $arr_msg;
}
/********** Function ga_module_check() END **********/


/***************************************************
 * Error Messege
 *--------------------------------------------------
 * @param string $msg
 ***************************************************/
function ga_set_errmsg($msg, $color_red = false){
    echo '<div style="background-color: rgb(255, 251, 204);';
    if ($color_red){
        echo 'color: rgb(255, 0, 0);';
    }
    echo '" id="message" class="updated fade"><p><strong>';
    if (is_array($msg)){
        foreacH($msg as $key => $val){
            $msg[$key] = __($val, 'g_analytics');
        }
        echo implode("<br>", $msg);
    }else{
        _e($msg, 'g_analytics');
    }
    echo '</strong></p></div>';
}
/********** Function ga_set_errmsg() END **********/

/***************************************************
 * Get the title of the post from the permalink.
 *--------------------------------------------------
 * @param string $permalink
 * @return String
 ***************************************************/
function ga_get_title($permalink){
    $siteurl = get_option('siteurl');
    $tmpPostID = url_to_postid($siteurl.$permalink);
    if (intval($tmpPostID) > 0){
        $title = get_the_title($tmpPostID);
    }else{
        $title = $permalink;
    }
    
    return $title;
}
/********** Function ga_get_title() END **********/

/***************************************************
 * The acquisition of date data
 *--------------------------------------------------
 * @param string $period
 * @param string $pause
 * @return array
 ***************************************************/
function ga_get_date($period, $pause = "-"){

    $format = 'Y'.$pause.'m'.$pause.'d';

    switch($period){
        case '3month':
            $date_start = date($format, strtotime('3 month ago'));
            $date_end = date($format);
            $max_results = 93;
            break;
            
        case 'lastmonth':
            $lastmonth = strtotime(date('Y/m/1') . ' -1 day');
            $date_start = date($format, mktime(0, 0, 0, date('m', $lastmonth), 1, date('Y', $lastmonth)));
            $date_end = date($format, mktime(0, 0, 0, date('m', $lastmonth) + 1, 0, date('Y', $lastmonth)));
            $max_results = 32;
            break;
            
        case 'thismonth':
            $date_start = strtotime(date('Y/m/1'));
            $date_start = date($format, $date_start);
            $date_end = date($format , strtotime($date_start . ' 1 month - 1 day'));
            $max_results = 32;
            break;
            
        case '1week':
            $date_start = date($format, strtotime('1 week ago'));
            $date_end = date($format);
            $max_results = 8;
            break;
            
        case 'yesterday':
            $date_start = date($format, strtotime('1 day ago'));
            $date_end = date($format, strtotime('1 day ago'));
            $max_results = 1;
            break;
            
        case 'today':
            
            $date_start = date($format);
            $date_end = date($format);
            $max_results = 1;
            break;
        
        default:
        case '1month':
            $date_start = date($format, strtotime('1 month ago'));
            $date_end = date($format);
            $max_results = 32;
            break;
            
    }
    
    return array('start_date'  =>  $date_start,
                 'end_date'    => $date_end,
                 'max_results' => $max_results,
                );
}
/********** Function ga_get_date() END **********/


/***************************************************
 * The making of Analytics data
 *--------------------------------------------------
 * @param array  $arr_data
 * @param array  $arr_metric
 * @param string $first_title
 * @return array
 ***************************************************/
function ga_get_make_analytics_data($arr_data, $arr_metric, $first_title = ''){
    array_unshift($arr_metric, GA_MY_LABEL);
    
    $arr_tmp = array();
    $arr_tile = array($first_title);
    $arr_result = array();
    if (is_array($arr_data)){
        foreach($arr_data as $row_key => $row_val){
            foreach($arr_metric as $metric_key => $metric_val){
                $arr_tmp[$row_key][$metric_val] = $row_val[$metric_key];
            }
        }
        
        foreach($arr_tmp as $row_key => $row_val){
            // MyTitle
            if (array_key_exists(GA_PAGE_TITLE, $row_val)){
                $arr_result['data'][$row_key][GA_PAGE_TITLE] = $row_val[GA_PAGE_TITLE];
            }
            
            // MyTitle
            if (array_key_exists(GA_MY_LABEL, $row_val)){
                $arr_result['data'][$row_key][GA_MY_LABEL] = $row_val[GA_MY_LABEL];
                if ($cnt == 0) $arr_tile[] = __(GA_MY_LABEL, 'g_analytics');
            }
            
            // Visits
            if (array_key_exists('ga:visits', $row_val)){
                $arr_result['data'][$row_key]['ga:visits'] = $row_val['ga:visits'];
                if ($cnt == 0) $arr_tile[] = __('Visits', 'g_analytics');
            }
            
            // Unique Users
            if (array_key_exists('ga:visitors', $row_val)){
                $arr_result['data'][$row_key]['ga:visitors'] = $row_val['ga:visitors'];
                if ($cnt == 0) $arr_tile[] = __('Unique Users', 'g_analytics');
            }
            
            // Pageviews
            if (array_key_exists('ga:pageviews', $row_val)){
                $arr_result['data'][$row_key]['ga:pageviews'] = $row_val['ga:pageviews'];
                if ($cnt == 0) $arr_tile[] = __('Pageviews', 'g_analytics');
            }
            
            // Average Pageviews
            if (array_key_exists('ga:visits', $row_val) && array_key_exists('ga:pageviews', $row_val)){
                if ($row_val['ga:pageviews'] == 0 || $row_val['ga:visits'] == 0){
                    $arr_result['data'][$row_key]['ga:averagePageviews'] = 0;
                }else{
                    $arr_result['data'][$row_key]['ga:averagePageviews'] = round( $row_val['ga:pageviews'] / $row_val['ga:visits'], 2 );
                }
                if ($cnt == 0) $arr_tile[] = __('Average Pageviews', 'g_analytics');
            }
            
            // Time on Site
            if (array_key_exists('ga:visits', $row_val) && array_key_exists('ga:timeOnPage', $row_val)){
                if ($row_val['ga:timeOnPage'] == 0 || $row_val['ga:visits'] == 0){
                    $arr_result['data'][$row_key]['ga:timeOnSite'] = '00:00:00';
                }else{
                    $arr_result['data'][$row_key]['ga:timeOnSite'] = gmdate('H:i:s', $row_val['ga:timeOnPage'] / $row_val['ga:visits']);
                }
                if ($cnt == 0) $arr_tile[] = __('Time on Site', 'g_analytics');
            }
            
            // Bounce Rate
            if (array_key_exists('ga:bounces', $row_val) && array_key_exists('ga:entrances', $row_val)){
                if ($row_val['ga:bounces'] == 0 || $row_val['ga:entrances'] == 0){
                    $arr_result['data'][$row_key]['ga:bounceRate'] = 0;
                }else{
                    $arr_result['data'][$row_key]['ga:bounceRate'] = round( $row_val['ga:bounces'] / $row_val['ga:entrances'], 4 ) * 100;
                }
                if ($cnt == 0) $arr_tile[] = __('Bounce Rate', 'g_analytics');
            }
            
            // Title
            if (array_key_exists(GA_PAGE_TITLE, $row_val)){
                $arr_result['data'][$row_key][GA_PAGE_TITLE] = $row_val[GA_PAGE_TITLE];
            }
            
            $cnt = 1;
        }
    }
    
    $arr_result['title'] = $arr_tile;
    
    return $arr_result;
    
}
/********** Function ga_get_make_analytics_data() END **********/

/***************************************************
 * Get Rss Data
 *--------------------------------------------------
 * @param array  $url
 * @param array  $row_num
 * @return array
 ***************************************************/
function ga_get_rss($url, $row_num){
    if (!function_exists("simplexml_load_file")) {
        return array();
    }
    
    $str = file_get_contents($url);
    $str = trim($str);
    $rss = @simplexml_load_string($str);
    if (!$rss) return array();
    
    $cnt = 0;
    $rss_list = array();
    foreach($rss->channel->item as $item){
        $rss_list[$cnt] = array(
            'title' => $item->title,
            'date'  => date("Y/m/d", strtotime($item->pubDate)),
            'link'  => $item->link,
        );
        
        $cnt++;
        if ($cnt >= $row_num) break;
    }
    
    return $rss_list;
}